@@ -24,7 +24,7 @@ from thinc.neural._classes.affine import _set_dimensions_if_needed
 import thinc.extra.load_nlp
 
 from .attrs import ID, ORTH, LOWER, NORM, PREFIX, SUFFIX, SHAPE
-from .errors import Errors
+from .errors import Errors, user_warning, Warnings
 from . import util
 
 try:
@@ -86,7 +86,7 @@ def with_cpu(ops, model):
     as necessary."""
     model.to_cpu()
 
-    def with_cpu_forward(inputs, drop=0.):
+    def with_cpu_forward(inputs, drop=0.0):
         cpu_outputs, backprop = model.begin_update(_to_cpu(inputs), drop=drop)
         gpu_outputs = _to_device(ops, cpu_outputs)
 
@@ -106,7 +106,7 @@ def _to_cpu(X):
         return tuple([_to_cpu(x) for x in X])
     elif isinstance(X, list):
         return [_to_cpu(x) for x in X]
-    elif hasattr(X, 'get'):
+    elif hasattr(X, "get"):
         return X.get()
     else:
         return X
@@ -142,7 +142,9 @@ class extract_ngrams(Model):
         # The dtype here matches what thinc is expecting -- which differs per
         # platform (by int definition). This should be fixed once the problem
         # is fixed on Thinc's side.
-        lengths = self.ops.asarray([arr.shape[0] for arr in batch_keys], dtype=numpy.int_)
+        lengths = self.ops.asarray(
+            [arr.shape[0] for arr in batch_keys], dtype=numpy.int_
+        )
         batch_keys = self.ops.xp.concatenate(batch_keys)
         batch_vals = self.ops.asarray(self.ops.xp.concatenate(batch_vals), dtype="f")
         return (batch_keys, batch_vals, lengths), None
@@ -297,7 +299,17 @@ def link_vectors_to_models(vocab):
     data = ops.asarray(vectors.data)
     # Set an entry here, so that vectors are accessed by StaticVectors
     # (unideal, I know)
-    thinc.extra.load_nlp.VECTORS[(ops.device, vectors.name)] = data
+    key = (ops.device, vectors.name)
+    if key in thinc.extra.load_nlp.VECTORS:
+        if thinc.extra.load_nlp.VECTORS[key].shape != data.shape:
+            # This is a hack to avoid the problem in #3853. Maybe we should
+            # print a warning as well?
+            old_name = vectors.name
+            new_name = vectors.name + "_%d" % data.shape[0]
+            user_warning(Warnings.W019.format(old=old_name, new=new_name))
+            vectors.name = new_name
+            key = (ops.device, vectors.name)
+    thinc.extra.load_nlp.VECTORS[key] = data
 
 
 def PyTorchBiLSTM(nO, nI, depth, dropout=0.2):
@@ -592,32 +604,27 @@ def build_text_classifier(nr_class, width=64, **cfg):
         )
 
         linear_model = build_bow_text_classifier(
-            nr_class, ngram_size=cfg.get("ngram_size", 1), exclusive_classes=False)
-        if cfg.get('exclusive_classes'):
+            nr_class, ngram_size=cfg.get("ngram_size", 1), exclusive_classes=False
+        )
+        if cfg.get("exclusive_classes"):
             output_layer = Softmax(nr_class, nr_class * 2)
         else:
             output_layer = (
-                zero_init(Affine(nr_class, nr_class * 2, drop_factor=0.0))
-                >> logistic
+                zero_init(Affine(nr_class, nr_class * 2, drop_factor=0.0)) >> logistic
             )
-        model = (
-            (linear_model | cnn_model)
-            >> output_layer
-        )
+        model = (linear_model | cnn_model) >> output_layer
         model.tok2vec = chain(tok2vec, flatten)
     model.nO = nr_class
     model.lsuv = False
     return model
 
 
-def build_bow_text_classifier(nr_class, ngram_size=1, exclusive_classes=False,
-        no_output_layer=False, **cfg):
+def build_bow_text_classifier(
+    nr_class, ngram_size=1, exclusive_classes=False, no_output_layer=False, **cfg
+):
     with Model.define_operators({">>": chain}):
-        model = (
-            extract_ngrams(ngram_size, attr=ORTH) 
-            >> with_cpu(Model.ops,
-                LinearModel(nr_class)
-            )
+        model = with_cpu(
+            Model.ops, extract_ngrams(ngram_size, attr=ORTH) >> LinearModel(nr_class)
         )
         if not no_output_layer:
             model = model >> (cpu_softmax if exclusive_classes else logistic)
@@ -626,11 +633,9 @@ def build_bow_text_classifier(nr_class, ngram_size=1, exclusive_classes=False,
 
 
 @layerize
-def cpu_softmax(X, drop=0.):
+def cpu_softmax(X, drop=0.0):
     ops = NumpyOps()
 
-    Y = ops.softmax(X)
-
     def cpu_softmax_backward(dY, sgd=None):
         return dY
 
@@ -648,13 +653,60 @@ def build_simple_cnn_text_classifier(tok2vec, nr_class, exclusive_classes=False,
         if exclusive_classes:
             output_layer = Softmax(nr_class, tok2vec.nO)
         else:
-            output_layer = zero_init(Affine(nr_class, tok2vec.nO, drop_factor=0.0)) >> logistic
+            output_layer = (
+                zero_init(Affine(nr_class, tok2vec.nO, drop_factor=0.0)) >> logistic
+            )
         model = tok2vec >> flatten_add_lengths >> Pooling(mean_pool) >> output_layer
     model.tok2vec = chain(tok2vec, flatten)
     model.nO = nr_class
     return model
 
 
+def build_nel_encoder(embed_width, hidden_width, ner_types, **cfg):
+    # TODO proper error
+    if "entity_width" not in cfg:
+        raise ValueError("entity_width not found")
+    if "context_width" not in cfg:
+        raise ValueError("context_width not found")
+
+    conv_depth = cfg.get("conv_depth", 2)
+    cnn_maxout_pieces = cfg.get("cnn_maxout_pieces", 3)
+    pretrained_vectors = cfg.get("pretrained_vectors")  # self.nlp.vocab.vectors.name
+    context_width = cfg.get("context_width")
+    entity_width = cfg.get("entity_width")
+
+    with Model.define_operators({">>": chain, "**": clone}):
+        model = (
+            Affine(entity_width, entity_width + context_width + 1 + ner_types)
+            >> Affine(1, entity_width, drop_factor=0.0)
+            >> logistic
+        )
+
+        # context encoder
+        tok2vec = (
+            Tok2Vec(
+                width=hidden_width,
+                embed_size=embed_width,
+                pretrained_vectors=pretrained_vectors,
+                cnn_maxout_pieces=cnn_maxout_pieces,
+                subword_features=True,
+                conv_depth=conv_depth,
+                bilstm_depth=0,
+            )
+            >> flatten_add_lengths
+            >> Pooling(mean_pool)
+            >> Residual(zero_init(Maxout(hidden_width, hidden_width)))
+            >> zero_init(Affine(context_width, hidden_width))
+        )
+
+        model.tok2vec = tok2vec
+
+    model.tok2vec = tok2vec
+    model.tok2vec.nO = context_width
+    model.nO = 1
+    return model
+
+
 @layerize
 def flatten(seqs, drop=0.0):
     ops = Model.ops
